//+------------------------------------------------------------------+
//|                            correlations short and long timeframe |
//|                                              corr_short_long.mq4 |
//|                                                             Adeo |
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
//| v01 20111030 initial release
//|
//|
//|
//|
//|
//|
//+------------------------------------------------------------------+
//
//
//
//
//
#property copyright "adeo"
#property link      "adeomercatus@yahoo.com"

#include <stdlib.mqh>
#include <stderror.mqh>

extern int     shortPeriods     = 20;
extern int     shortTF          = 60;
extern double  shortCorr        = 0.8;
extern int     longPeriods      = 20;
extern int     longTF           = 1440;
extern double  longCorr         = 0.8;
extern int     wideCount        = 5;
extern bool    showMessages     = false;
extern int     debug            = 0;

int      symbolCount;
string   symbols[];
string   symbol;
string   prefix      = "NA";
string   suffix      = "NA";
int      oldBars;

int init() {
   shortPeriods = MathMax(shortPeriods,1);
   shortTF = MathMin(MathMax(shortTF,1),1440);
   longPeriods = MathMax(longPeriods,1);
   longTF = MathMin(MathMax(longTF,1),1440);
   debug = MathMax(debug,0);
   getSymbolsList();   
   symbolCount = ArraySize(symbols);
   if (symbolCount<1) return(-1);
   getPrefixSuffix(prefix, suffix);

   return(0);
}
int deinit() {
   return(0);
}


int start() {
   int Total;
   int i,j,k;
   
   
   if (oldBars != Bars)
   {
      string sComment   = "";
      string NL         = "\n";
      sComment = sComment + WindowExpertName()+ "  Balance=" + DoubleToStr(AccountBalance(),0)+ "  Equity=" + DoubleToStr(AccountEquity(),0) + NL;
      sComment = sComment + "Current Server DateTime = " + TimeToStr(TimeCurrent(), TIME_DATE|TIME_SECONDS) + NL;
      sComment = sComment + "   Correlation shortTF  " +  shortTF + "   over " + shortPeriods + " periods" + "   Correlation longTF  " +  longTF + "   over " + longPeriods + " periods" +NL;
      k=0;
      for(i=0; i<symbolCount; i++)
      {
        for(j=0; j<symbolCount; j++) 
        {
         if (i!=j)
         {
            if (debug>0) Print("loopSymbols i="+i+" j="+j);
            double corrShort = Correlation(symbols[i],symbols[j],shortPeriods,shortTF);
            double corrLong  = Correlation(symbols[i],symbols[j],longPeriods, longTF);
            if ((MathAbs(corrShort) >= shortCorr) && (MathAbs(corrLong) >= longCorr))
            {
               sComment = sComment + " ..." + symbols[i] + " " + symbols[j] + " " + DoubleToStr(corrShort,2)+ " " + DoubleToStr(corrLong,2);
               k++;
            }
            if (k>=wideCount)
            {
               sComment = sComment + NL;
               k=0;
            }
         }
         
        }
      }
   Comment(sComment);
   }
   oldBars = Bars;
   return(0);
}


//
//
//
//
//

double Correlation(string Instrument1, string Instrument2, int maPeriod, double tf)
{
   if((iBars(Instrument1,tf)<=maPeriod) || (iBars(Instrument2,tf)<=maPeriod)) return(0);
   if ((tf < 0) || (tf>43200)) return (0);
   int k;
   double x, y;
   double sumX=0;
   double sumY=0;
   double cor1, cor2, cor3;
   for(k=0; k<maPeriod; k++)
   {
      sumX += iClose(Instrument1,tf,k);
      sumY += iClose(Instrument2,tf,k);
   }
   if (maPeriod==0)
   {
      if (debug>32) Print("maPeriod=0 !!!");
      return(0);
   }   
   sumX/=maPeriod;
   sumY/=maPeriod;
   cor1=0;cor2=0;cor3=0;
   for(k=0; k<maPeriod; k++)
   {
      x = iClose(Instrument1,tf,k);
      y = iClose(Instrument2,tf,k);
      cor1+=(x-sumX)*(y-sumY);
      cor2+=(x-sumX)*(x-sumX);
      cor3+=(y-sumY)*(y-sumY);
   }
   if (debug>128) Print(DoubleToStr(cor1,2)+ DoubleToStr(cor2,2)+DoubleToStr(cor3,2));

   return(cor1/MathSqrt(cor2)/MathSqrt(cor3));
}


//+------------------------------------------------------------------+
//|    following routines coded by mladen                            |
//+------------------------------------------------------------------+
//
//
//
//
//

#import "kernel32.dll"
    int _lclose (int);
    int _lopen  (string,int);
    int _llseek (int,int,int);
    int _lread  (int,string,int);
#import
#define sectorSize  1936
#define HFILE_ERROR -1


//
//
//
//
//

void getSymbolsList()
{
   ArrayResize(symbols,0);
   int fileHandle = _lopen(TerminalPath()+"\\history\\"+AccountServer()+"\\symbols.raw",0);
   if (fileHandle == HFILE_ERROR) return;
    
   //
   //
   //
   //
   //

   for(int i=0;; i++)
   {
      _llseek(fileHandle, i * sectorSize, 0);
      string symbolName = "123456789012";       
                        if (_lread (fileHandle, symbolName, 12) != 12) break;
             symbolName = StringSubstr(symbolName, 0);
   if (debug>64) Print(symbolName);

             //
             //
             //
             //
             //

      ArrayResize(symbols,i+1); symbols[i] = StringTrimLeft(StringTrimRight(symbolName));
   } 
   _lclose(fileHandle);
}

//
//
//
//
//

bool findSymbol(string findWhat)
{
   int  limit = ArraySize(symbols);
         for(int i=0; i<limit; i++) if (findWhat==symbols[i]) break;
   return(i<limit);      
}
void getPrefixSuffix(string& prefix, string& suffix)
{ 
   getSymbolsList();   
   int limit = ArraySize(symbols);
   if (limit<1) return;
         prefix="";
         suffix="";
   
   //
   //
   //
   //
   //
   
   for(int i=0; i<limit; i++)
   {
      string symbolName = symbols[i];
             int pos = StringFind(symbolName,"EURUSD",0);
             if (pos > -1)
             {
                if (pos>0)                         prefix = StringSubstr(symbolName,0,pos);
                if ((pos+6)<StringLen(symbolName)) suffix = StringSubstr(symbolName,(pos+6),0);
                break;
             }
   } 
}

//
//
//
//
//
string StringUpperCase(string str)
{
   string   s = str;
   int      lenght = StringLen(str) - 1;
   int      char;
   
   while(lenght >= 0)
      {
         char = StringGetChar(s, lenght);
         
         //
         //
         //
         //
         //
         
         if((char > 96 && char < 123) || (char > 223 && char < 256))
                  s = StringSetChar(s, lenght, char - 32);
          else 
              if(char > -33 && char < 0)
                  s = StringSetChar(s, lenght, char + 224);
         lenght--;
   }
   return(s);
}


//----------------------------------------------------------------------------------------
//       messages handling
//----------------------------------------------------------------------------------------
//
//
//
//
//
//

bool   msg.errState[];
string msg.errNames[];
string msg.messageText[10];
color  msg.messageColor[10];
int    msg.lastMessage=0;

//
//
//
//
//

void checkError(string what,int err)
{
   showMessage(what);
   if(err!=0)
         showMessage("error occured :"+ErrorDescription(err),Red);

}

//
//
//
//
//

void showTwoStateMessage(string name, string message, bool state)
{
   for(int i=ArraySize(msg.errNames)-1; i>=0; i--) if (msg.errNames[i]==name) break;
   if (i==-1)
      {
         int size = ArraySize(msg.errNames)+1;
         i = size-1;
            ArrayResize(msg.errNames,size); msg.errNames[i] = name;
            ArrayResize(msg.errState,size); msg.errState[i] = true;
      }

   //
   //
   //
   //
   //

   if (msg.errState[i]!= state)
   {   
      msg.errState[i] = state;
      if (state==false)
            showMessage(message,Red);
      else  showMessage(message,Green);
   }      
}

//
//
//
//
//

void showMessage(string text, color theColor=Gray)
{
   if(!showMessages) { Print(text); return; }
   
   //
   //
   //
   //
   //
   
   if(msg.lastMessage>9)
   {
      for(int i=0; i<9; i++)
      {
         msg.messageText[i] =msg.messageText[i+1];
         msg.messageColor[i]=msg.messageColor[i+1];
      }
      msg.lastMessage = 9;
   }

   //
   //
   //    set message que text and color
   //
   //
   
      msg.messageText[msg.lastMessage]  = text+" - "+TimeToStr(TimeCurrent(),TIME_DATE|TIME_SECONDS);
      msg.messageColor[msg.lastMessage] = theColor;
   
   //
   //
   //
   //
   //
      
   for(i=0; i<=msg.lastMessage; i++)
   {
      string name = "msg.que"+i;
      if (ObjectFind(name) == -1)
      {
         ObjectCreate(name,OBJ_LABEL,0,0,0);
            ObjectSet(name,OBJPROP_CORNER,3);
            ObjectSet(name,OBJPROP_XDISTANCE,5);
            ObjectSet(name,OBJPROP_YDISTANCE,5+14*i);
      }
      ObjectSetText(name,msg.messageText[i],9,"Arial",msg.messageColor[i]);
   }
   msg.lastMessage++;
   WindowRedraw();
}


